from __future__ import print_function
from .logsetup import UserLogger
import requests
import ssl
import sys
from infi.credentials_store import CLICredentialsStore, Credentials
import pickle
import traceback
from docopt import docopt

requests.packages.urllib3.disable_warnings()

try:
    _create_unverified_https_context = ssl._create_unverified_context
except AttributeError:
    # Legacy Python that doesn't verify HTTPS certificates by default
    pass
else:
    # Handle target environment that doesn't support HTTPS verification
    ssl._create_default_https_context = _create_unverified_https_context


log = UserLogger(__name__)


def handle(signum, frame):
    sys.exit(1)


def get_function_name():
    return traceback.extract_stack(None, 2)[0][2]


def load_cookie(infinibox, username, hostname):
    try:
        with open('/tmp/iguard_tools_cookie_{}_{}'.format(username, hostname), 'rb') as f:
            return pickle.load(f)
    except Exception:
        log.debug("Cookie not found.")
    return None


def save_cookie(infinibox, username, hostname):
    cookie = None
    try:
        with open('/tmp/iguard_tools_cookie_{}_{}'.format(username, hostname), 'wb') as f:
            pickle.dump(infinibox.api._session.cookies.copy(), f)
    except Exception as e:
        log.debug("Failed to save cookie")
        log.debug(repr(e))


def my_login(infinibox, username, hostname):
    cookie = load_cookie(infinibox, username, hostname)
    if not cookie:
        log.debug("Cookie not found. Logging in.")
        infinibox.login()
        cookie = save_cookie(infinibox, username, hostname)
    else:
        log.debug("Loaded saved cookie")
        infinibox.api._session.cookies = cookie
        try:
            infinibox.get_version()
            # It didn't use the cookie, but looged in
            if infinibox.is_logged_in():
                log.debug("Logged in, cookie expired. Saving.")
                save_cookie(infinibox, username, hostname)
        except Exception:
            log.debug("Cookie not accepted")
            infinibox.api._session.cookies.clear()
            infinibox.login()
            save_cookie(infinibox, username, hostname)


class MyCLICrdentialsStore(CLICredentialsStore):
    def _get_file_folder(self):
        return ".infi.credentials_store"

    def authenticate(self, key, credentials):
        return True

    def ask_credentials_prompt(self, key):
        print('Connecting to system ' + str(key))


def delete_credentials(store):
    print("Deleting all credentials")
    store.reset_credentials()


def save_credentials(store, hostname, username, password):
    credentials = Credentials(username, password)
    store.set_credentials(hostname, credentials)
    print("Stored credentials for host {}".format(hostname))


def validate_config(arguments, store):
    config = {}
    if arguments['--help']:
        docopt(__doc__, '-h')

    for key, value in arguments.items():
        config[key[2:]] = value
    if not config['delete']:
        if not config['username'] or not config['password']:
            credentials = store.get_credentials(config['hostname'])
            config['username'] = credentials.get_username()
            config['password'] = credentials.get_password()
    if not config['delete'] and not config['save']:
        try:
            dde_id = int(config['dde'])
            if dde_id not in [1, 2]:
                raise Exception
            config['dde'] = dde_id
        except Exception as e:
            raise Exception('DDE ID should be 1 or 2. Exiting.')
    return config


class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


def print_err(*args, **kwargs):
    print(f'{bcolors.FAIL}{" ".join([a for a in args])}{bcolors.ENDC}', file=sys.stderr, **kwargs)


def is_task_finished(infinibox, dde_id, task_id):
    task_url = "/api/dde/{}/api/asynctasks/{}".format(dde_id, task_id)
    get = infinibox.api.get(task_url)
    status = get.get_result().get('status')
    return status != 'RUNNING'
